//=====================================================
//      Name:            Gear_Math.java
//      Project:         Math Stuff
//      Version:         0.1
//
//      Author:           Joshua Gulick, Orphan Technologies, Inc.
//      Date:               6/09/2000
//      Comment:       Calculates rpm and torque of some simple gear designs
//=====================================================


//===========imports===================================
import ij.*;
import ij.gui.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import java.awt.*;
import ij.plugin.PlugIn;
import ij.text.*;

//===========source====================================
public class Gear_Math implements PlugIn {

public void run(String arg) {
    double g1 = 12, g2 = 34560, g3 = 144, s1rpm = 2880, s1torque = 34560;
    GenericDialog gd = new GenericDialog("Enter Gear Info", IJ.getInstance());
    gd.addMessage("All gears have equally spaced teeth. The screw\n"
		+"threads are spaced equal to the gear teeth");
    gd.addNumericField(" RPM of drive screw ", s1rpm, 1);
    gd.addNumericField(" torque (ft./lbs.) of drive screw ", s1torque, 1);
    gd.addNumericField(" # teeth on gear 1 ", g1, 1);
    gd.addNumericField(" # teeth on gear 2 ", g2, 1);
    gd.addNumericField(" # teeth on gear 3 ", g3, 1);

    gd.showDialog();
        if (gd.wasCanceled())
              return;
    s1rpm =  gd.getNextNumber();
    s1torque =  gd.getNextNumber();
    g1 =  gd.getNextNumber();
    g2 =  gd.getNextNumber();
    g3 =  gd.getNextNumber();

    double g1rpm, g1torque, g2rpm, g2torque, g3rpm, g3torque;
    g1rpm = s1rpm/g1;
    g1torque = s1torque*(g1/s1rpm);
    g2rpm = g1rpm*(g1/g2);
    g3rpm = g2rpm*(g2/g3);
    g2torque = g1torque*(g2/g1);
    g3torque = g2torque*(g3/g2);

    TextWindow tw = new TextWindow("Gear Math", "", 700, 300);
    tw.append("A screw gear rotating at " + s1rpm + " rpm and having a torque of " + s1torque + " ft./lbs.");
    tw.append("drives a gear with " + g1 + " teeth, causing it to rotate at " + IJ.d2s(g1rpm,1) + " rpm with a torque of " + g1torque + " ft./lbs.");
    tw.append("This drives a second gear with " + g2 + " teeth, causing it to rotate at " + IJ.d2s(g2rpm,1) + " rpm with a torque of " + g2torque + " ft./lbs.");
    tw.append("The second gear, in turn, drives a third gear with " + g3 + " teeth which rotates at " + IJ.d2s(g3rpm,1) + " rpm and has a torque of " + g3torque + " ft./lbs.");
    }
}
